library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity cfctrl is
	Port ( rst : in std_logic;
           clk : in std_logic;
           cf_cs : in std_logic;       -- pushes state machine out of idle state
		   cf_rdy : in std_logic;      -- compactflash ready signal

		   --cf_go_addr : out std_logic;
		   cf_busy : out std_logic;    -- not finished full read sequence
		   cf_ce0 : out std_logic;      -- cf chip enable 0
		   cf_ce1 : out std_logic;      -- cf chip enable 1
		   cf_oe : out std_logic;      -- cf output enable
		   cf_read : out std_logic;    -- data available as of this cycle
		   cf_bram_addr : out std_logic_vector( 7 downto 0 )
	);
end cfctrl;

architecture Behavioral of cfctrl is

signal state : std_logic_vector( 2 downto 0 );
--signal go_addr : std_logic;
signal busy : std_logic;
signal ce : std_logic;
signal oe : std_logic;
signal reading : std_logic;
signal word_counter : std_logic_vector( 7 downto 0 );

begin

process( rst, clk, cf_cs, cf_rdy )
begin
	if rst = '1' then
		state <= "000";
		--go_addr <= '0';
		ce <= '0';
		oe <= '0';
		reading <= '0';
		busy <= '0';
		word_counter <= X"00";
	elsif clk'event and clk = '1' then -- positive clk edge
		if state = "000" and cf_cs = '1' then
			-- set address
			state <= "001";
			--go_addr <= '1';
			busy <= '1';
		elsif state = "001" then
			-- chip enable and output enable 
			state <= "010";
			--go_addr <= '1';
			ce <= '1';
			oe <= '1';
		elsif state = "010" and cf_rdy = '1' then
			-- done waiting, ready to receive data
			state <= "011";
			reading <= '1';
			word_counter <= X"00";
		elsif state = "011" then
			if word_counter = X"FF" then
				-- done retrieving sector
				word_counter <= X"00";
				state <= "100";
				ce <= '0';
				oe <= '0';
			else
				word_counter <= word_counter + 1;
			end if;
		elsif state = "100" then
			if word_counter = X"04" then
				state <= "000";
				--go_addr <= '0';
				ce <= '0';
				oe <= '0';
				reading <= '0';
				busy <= '0';
				word_counter <= X"00";
			else
				word_counter <= word_counter + 1;
			end if;
		end if;
	end if;

	--cf_go_addr <= go_addr;
	cf_busy <= busy;
	cf_ce0 <= not ce;
	cf_ce1 <= not ce;
	cf_oe <= oe;
	cf_read <= reading;
	cf_bram_addr <= word_counter;
end process;
		
end Behavioral;
