library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity opb_xsb300_cf is
  generic (
    C_OPB_AWIDTH    : integer := 32;
    C_OPB_DWIDTH    : integer := 32;
    C_BASEADDR      : std_logic_vector := X"2000_0000";
    C_HIGHADDR      : std_logic_vector := X"2000_00FF"
    );

    Port ( 	OPB_Clk : in std_logic;
				OPB_Rst : in std_logic;
				OPB_ABus : in std_logic_vector (31 downto 0);
				OPB_BE : in std_logic_vector (3 downto 0);
				OPB_DBus : in std_logic_vector (31 downto 0);
				OPB_RNW : in std_logic;
				OPB_select : in std_logic;
				OPB_seqAddr : in std_logic;
				UIO_DBus : out std_logic_vector (31 downto 0);
				UIO_errAck : out std_logic;
				UIO_retry : out std_logic;
				UIO_toutSup : out std_logic;
				UIO_xferAck : out std_logic;
				PB_A : out std_logic_vector (19 downto 0);
				PB_WE_N : out std_logic;
				PB_OE_N : out std_logic;
				PB_D : inout std_logic_vector (15 downto 0);
                CS0_N : out std_logic;
                CS1_N : out std_logic;
                IOCS16_N : in std_logic;
                IORDY : in std_logic;
                INTRQ : in std_logic
	 		   );
end opb_xsb300_cf;

architecture Behavioral of opb_xsb300_cf is

component pad_io 
    Port ( clk : in std_logic;
           rst : in std_logic;
           PB_A : out std_logic_vector(19 downto 0);
           PB_WE_N : out std_logic;
           PB_OE_N : out std_logic;
           PB_D : inout std_logic_vector(15 downto 0);
           CS0_N : out std_logic;
           CS1_N : out std_logic;
           IOCS16_N : in std_logic;
           IORDY : in std_logic;
           INTRQ : in std_logic;
           pb_addr : in std_logic_vector(19 downto 0);
           pb_wr : in std_logic;
           pb_rd : in std_logic;
           pb_dread : out std_logic_vector(15 downto 0);
           pb_dwrite : in std_logic_vector(15 downto 0);
           cf_cs_pad0 : in std_logic;
           cf_cs_pad1 : in std_logic;
           cf_rdy : out std_logic;
		   cf_cs16 : out std_logic;
		   cf_intr : out std_logic
);
end component;

signal cs : std_logic;

signal addr: std_logic_vector( 19 downto 0 );

signal xfer : std_logic;

signal oe_timer :std_logic_vector(8 downto 0) := "000000000";

signal state :std_logic;





signal cf_rdy : std_logic;
signal cf_addr : std_logic_vector( 19 downto 0 );
signal cf_data : std_logic_vector( 15 downto 0 );
signal cf_ce0 : std_logic :='1';
signal cf_ce1 : std_logic :='1';
signal cf_oe : std_logic :='0';
signal cf_intr : std_logic;
signal cf_cs16 : std_logic;


begin

pad_io1 : pad_io port map (
	clk => OPB_Clk,
	rst => OPB_Rst,
	PB_A => PB_A,
	PB_WE_N => PB_WE_N,
	PB_OE_N => PB_OE_N, 
	PB_D => PB_D,
	CS0_N => CS0_N,
	CS1_N => CS1_N,
	IOCS16_N => IOCS16_N,
	IORDY => IORDY,
	INTRQ => INTRQ,
	pb_addr => cf_addr,
	pb_rd => cf_oe,
	pb_wr => '0',
	pb_dread => cf_data,
	pb_dwrite => "0000000000000000",
	cf_cs_pad0 => cf_ce0,
	cf_cs_pad1 => cf_ce1,
	cf_rdy => cf_rdy,
	cf_cs16 => cf_cs16,
	cf_intr => cf_intr
);




process( OPB_Clk, OPB_Rst )
begin
	if OPB_Rst = '1' then
		cs <= '0';
	elsif OPB_Clk'event and OPB_Clk = '1' then
		if OPB_ABus(31 downto 20) = X"008" then
			cs <= OPB_select;
        else
            cs <= '0';
		end if;
	end if;
end process;


process(OPB_Clk)
begin
	if OPB_Rst = '1' then
		oe_timer <= "000000000";
		cf_ce0<='0';
		cf_ce1<='0';
		cf_oe<= '0';
	elsif OPB_Clk'event and OPB_Clk = '1' then
		if oe_timer="100000000" then
			cf_oe <= not cf_oe;
		--elsif oe_timer = "000000000" then
		--	cf_oe <= not cf_oe;
		--elsif oe_timer = "000000011" then
		--	cf_ce0 <= not cf_ce0;
		--	cf_ce1 <= not cf_ce1;
		--elsif oe_timer="011111101" then
		--	cf_ce0 <= not cf_ce0;
		--	cf_ce1 <= not cf_ce1;
		
		end if;
		oe_timer <= oe_timer +1;
	end if;
end process;

process( OPB_Clk, OPB_Rst )
begin
	if OPB_Rst = '1' then
		state <= '0';
	elsif OPB_Clk'event and OPB_Clk = '1' then
		  state <= (not state) and cs;
		  if state='1' then
			  
			UIO_DBus <= cf_data & cf_data;
        	xfer <= '1';
          else
            UIO_DBus <= X"00000000";
            xfer <='0';
          end if;
	end if;
cf_addr( 19 downto 17 ) <= "101";
cf_addr( 10 downto 0 )  <= " 0110100100";
cf_addr(16 downto 11) <= "000000";

-- tie unused to ground
UIO_errAck <= '0';
UIO_retry <= '0';
UIO_toutSup <= '0';
UIO_xferAck <= xfer;

end process;
end Behavioral;
